#ifndef DEF_HEADER_EXO_1
#define DEF_HEADER_EXO_1

#include "../test.h"

typedef struct _Cellule Cellule, *Liste;
struct _Cellule
{
	int valeur;
	Liste suivant;
};

/*
On considère que la Liste vide est repéresentée par NULL
*/

Liste Liste_Create(int valeur);
/*
Cette fonction doit renvoyer une structure Liste allouée dynamiquement
Dont la valeur sera "valeur"
*/

void Cellule_Free(Liste *liste);
/*
Cette fonction libère uniquement une Cellule* précise
Par convention la Cellule* vaut NULL après cet appel
On doit prendre en compte le cas où l'utilisateur essaierait de libérer une Cellule* vide
*/

void Liste_Free(Liste *liste);
/*
Cette fonction libère une Cellule* et ses suivantes
Par convention la Cellule* vaut NULL après cet appel
On doit prendre en compte le cas où l'utilisateur essaierait de libérer une Cellule* vide
*/

int Liste_Ajout_EnTete(Liste *liste, int valeur);
/*
Cette fonction ajoute un élément en tête de la liste commençant par la Cellule*
*/

int Liste_Ajout_EnFin(Liste *liste, int valeur);
/*
Cette fonction ajoute un élément en fin de la liste commençant par la Cellule*
*/

int Liste_Supprimer_EnTete(Liste *liste);
/*
Cette fonction permet la suppression de l'élement en tête de la liste commençant par la Cellule*
*/

int Liste_Supprimer_EnFin(Liste *liste);
/*
Cette fonction permet la suppression de l'élement en fin de la liste commençant par la Cellule*
*/

int Liste_Taille(Liste liste);
/*
Cette fonction donne la taille de la liste commençant à une Cellule* choisie
*/

int Liste_Somme(Liste liste);
/*
Cette fonction renvoie la somme de toutes les valeurs de la liste chainée
*/

int Liste_Concatener(Liste *liste, Liste *suite);
/*
Cette fonction concatène deux Cellule* avec suppression de la seconde
*/

int Liste_Copier(Liste *liste, Liste source);
/*
Cette fonction permet d'obtenir une copie d'une Cellule* source
Les modifications de source ne se répercutent pas sur *liste
*/

void Liste_Inverser(Liste *liste);
/*
Permet d'inverser une liste chainée (commence par le dernier et le précédent devient le suivant)
*/

Liste Liste_AdditionValeur(Liste liste, int valeur);
/*
On renverra une copie de la Cellule où
On additionne valeur à chaque valeur de la liste chainée
Cas particulier de Liste_OperationValeur servant d'échauffement
*/

Liste Liste_Addition(Liste un, Liste deux);
/*
On additionne unes à unes les valeurs des deux listes sans modifier les listes
Si une des listes n'a plus d'élément on additionne ceux de l'autre à 0
Cas particulier de Liste_Operation servant d'échauffement
*/

Liste Liste_OperationValeur(Liste liste, int op(int a, int b), int valeur);
/*
On renverra une copie de la Cellule où
On applique une opération à chaque valeur de la liste chainée
*/

Liste Liste_Operation(Liste un, int op(int a, int b), Liste deux);
/*
On applique l'operation op sur les valeurs unes à unes des deux listes sans modifier les listes
Si une des listes n'a plus d'élément on considère qu'on effectue l'opération  sur l'élément neutre de l'opération (En gros ça ne change rien à la valeur existante)
*/

Liste Liste_Generation(int debut, int fin, int fct(int n));
/*
Permet de générer une liste chainée d'après une fonction prennant en argument un indice allant de debut à fin (inclus)
*/

int fibonacci(int n);
/*
Deux choix sont offerts:
-En intératif
-Faire une sous fonction récursive
Dans les deux cas vous devez créer une fonction relativement performante
*/

int multiplier(int a, int b);
/*
Sera utilisée en tant qu'opération pour les tests de Liste_OperationValeur et Liste_Operation
*/

int Liste_CalculePolynome(Liste coeff, Liste indice, int x);
/*
On considère un polynôme défini par deux listes chainées (On pourrait aussi faire une structure Polynome)
coeff est la "liste des coefficients"
indice est la "liste des indices"
C'est-à-dire qu'on considère le polynôme comme la somme des
coeff(i)*(x^indice(i)) où i est l'emplacement dans la liste
Si coeff est plus longue que indice, on considèrera que des 1 comblent la liste 1*x^indice(i) = x^indice(i)
Si indice est plus longue que coeff on considèrera que des 0 comblent la liste coeff(i)*x^0 = coeff(i)
La liste indice peut ne pas être triée et posséder des doublons
Cette fonction donne la valeur du polynôme en x
*/

int Liste_EstPalindrome(Liste liste);
/*
Indique si la Cellule est un palindrome ou non
exemple:
1->2->3->2->1 est un palindrome
1->2->4->5->1 n'est pas un palindrome car (2 != 5)
On considèrra qu'une liste vide est un palindrome
*/

Liste Liste_Create_ByString(const char *chaine);
/*
Permet de créer un liste chainée dont les valeurs sont listees de la chaine de caractère passée en argument
*/

Liste Liste_Code(Liste liste, Liste cle);
/*
Permet d'obtenir une liste chainée grâce à une clé et une formule étrange
Il est fortement déconseillé de modifier cette fonction
*/

Liste Liste_Decode(Liste liste, Liste cle);
/*
Opération inverse de la fonction Liste_Code permettant de connaître le message codé
*/

/*
Fonctions utilisées dans Liste_Code:
Mince, il va falloir les recoder elles aussi
*/

int XOR(int a, int b);

void Liste_AppliquerOperation(Liste un, int op(int a, int b), Liste deux);
/*
On modifie les valeurs de "un" unes à unes en leur appliquant une opération avec les valeurs de "deux"
*/

void Liste_AppliquerOperationValeur(Liste un, int op(int a, int b), int valeur);
/*
On modifie les valeurs de "un" unes à unes en leur appliquant une opération avec valeur
*/

Liste Liste_Dernier(Liste liste);
/*
Donne la dernière Cellule de la liste chainée
*/

//Fonctions debug cadeau:

void Liste_AfficherString(Liste liste);
/*
Permet d'afficher une liste chainée sous forme d'une chaine de caractères
*/

void Liste_Afficher(Liste liste);
/*
Fonction affichant les entiers de la liste chainées
*/

#endif
