#ifndef DEF_HEADER_EXO_2
#define DEF_HEADER_EXO_2

#include "../test.h"

typedef struct _Matrice Case, *Matrice;
struct _Matrice
{
	int valeur;
	Matrice nextLigne;
	Matrice nextColonne;
};

/*
On considère qu'une matrice n'existe pas quand elle vaut NULL
*/

Matrice Matrice_Create(int valeur);
/*
Cette fonction doit renvoyer une structure Matrice allouée dynamiquement
Dont la valeur sera "valeur"
*/

void Case_Free(Matrice *mat);
/*
Cette fonction libère uniquement une Matrice précise
Par convention la Matrice vaut NULL après cet appel
On doit prendre en compte le cas où l'utilisateur essaierait de libérer une Matrice inexistante
*/

void Matrice_FreeLigne(Matrice *mat);
/*
Cette fonction libère uniquement une Matrice avec ses lignes suivantes
Par convention la Matrice vaut NULL après cet appel
On doit prendre en compte le cas où l'utilisateur essaierait de libérer une Matrice inexistante
*/

void Matrice_Free(Matrice *mat);
/*
Cette fonction libère uniquement une Matrice avec ses lignes et colonnes suivantes
Par convention la Matrice vaut NULL après cet appel
On doit prendre en compte le cas où l'utilisateur essaierait de libérer une Matrice inexistante
*/

int Matrice_Ajouter_Ligne(Matrice *mat);
/*
Cette fonction ajoute une ligne à la fin de la Matrice
Pour le moment la ligne ajoutée ne sera composée que de 0
*/

int Matrice_Ajouter_Colonne(Matrice *mat);
/*
Cette fonction ajoute une ligne à la fin de la Matrice
Pour le moment la ligne ajoutée ne sera composée que de 0
*/

int Matrice_Lignes(Matrice mat);
/*
Donne le nombre de lignes d'une matrice
*/

int Matrice_Colonnes(Matrice mat);
/*
Donne le nombre de lignes d'une matrice
*/

Matrice Matrice_CreateBySize(int lignes, int colonnes);
/*
Créée une matrice de taille lignesXcolonnes
*/

Matrice Matrice_CreateBySizeAndFunction(int lignes, int colonnes, int fonction(int ligne_i, int colonne_i));
/*
Permet de générer une Matrice grâce à une fonction donnée prenant en arguments le numéro de ligne et le numéro de colonne
Attention la première ligne est la ligne 1
*/

int identite(int ligne, int colonne);
/*
Fonction permettant la création de la matrice identité
*/

int Matrice_Edit(Matrice mat, int ligne, int colonne, int new_valeur);
/*
Cette fonction doit modifier la valeur d'indice (ligne,colonne) par new_valeur
*/

int Matrice_Supprimer_Ligne(Matrice *mat, int ligne);
/*
Permet de supprimer la ligne d'une matrice par son indice
*/

int Matrice_Supprimer_Colonne(Matrice *mat, int colonne);
/*
Permet de supprimer la colonne d'une matrice par son indice
*/

Matrice Matrice_GetCopie(Matrice mat);
/*
Permet d'obtenir une copie d'une Matrice
Les modifications de la copie ne doivent pas affecter la matrice de base
*/

/*
PARTIE PLUS MATHEMATIQUE
Maintenant que nos fonctions de création et édition de matrices fonctionnent
On va pouvoir s'attaquer à la partie qui fera aussi réviser l'algèbre
*/

int Matrice_Trace(Matrice mat);
/*
Retourne la trace d'une matrice
(Somme des éléments de la diagonale)
Par convention si une matrice n'est pas carrée on renverra la valeur 0
*/

int Matrice_Additionner(Matrice *mat, Matrice ajout);
/*
Cette fonction serait équivalente à mat += ajout
Elle fait l'addition de deux Matrices
*/

int Matrice_Multiplier(Matrice *mat, Matrice ajout);
/*
Cette fonction serait équivalente à mat *= ajout
Elle fait la multiplication de deux Matrices
Rappel: si C = AB, Ci,j = Somme(k allant de 1 à n)(Ai,k * Bk,j)
*/

int Matrice_Determinant(Matrice mat);
/*
Cette fonction donne le déterminant d'une matrice
Conseil: utilisez par exemple le développement selon une ligne ou une colonne
*/

int Matrice_Transposer(Matrice *mat);
/*
Cette fonction permet de transposer une matrice
*/

Matrice Matrice_GetComatrice(Matrice mat);
/*
Cette fonction renvoie la Comatrice d'une Matrice
C'est-à-dire la matrice des cofacteurs (mineurs) de la Matrice en argument
*/

void Matrice_Multiplier_ParScalaire(Matrice mat, int scalaire);
/*
Permet de multiplier une matrice par un scalaire
*/

void Matrice_Diviser_ParScalaire(Matrice mat, int scalaire);
/*
Permet de diviser une matrice par un scalaire
*/

Matrice Matrice_GetInverse(Matrice mat);
/*
Renvoie la matrice inverse de la matrice donnée en argument si la matrice est inversible
Pour cet exercice on prendre des matrices dont l'inverse est à valeurs entières
Si ça vous amuse vous pouvez aussi gérer les valeurs avec des double par exemple pour inverse toutes les matrices
Mais aussi par exemple faire que les matrices possèdent une valeur numérateur et une valeur dénominateur pour avoir des fractions
*/

int pascal(int n, int k);
/*
Fonction permettant la construction d'une matrice représentant le triangle de Pascal
La matrice sera triangulaire inférieure
*/

int Matrice_Concatener_Ligne(Matrice *mat, Matrice *ajout);
/*
Permet de concatener une matrice comme un esemble de nouvelles lignes d'une matrice
Il faut que les deux matrices aient le même nombre de colonnes pour être concaténées
La seconde Matrice sera nulle après appel
*/

int Matrice_Concatener_Colonne(Matrice *mat, Matrice *ajout);
/*
Même principe que Matrice_Concatener_Ligne sauf que la concaténation s'applique aux colonnes
*/

int Matrice_EstEgal(Matrice mat, Matrice autre);
/*
Renvoie 1 si les matrices sont égales et 0 sinon
*/

int Matrice_Symetrique(Matrice mat);
/*
Renvoie 1 si la matrice est symétrique et 0 sinon
*/

int Matrice_Antisymetrique(Matrice mat);
/*
Renvoie 1 si la matrice est symétrique et 0 sinon
*/

Matrice Matrice_Code(Matrice mat, Matrice cle);
/*
Renvoie une matrice depuis laquelle il est possible de retrouver mat grâce à Matrice_Decode
Dans le but de rendre la nouvelle matrice illisible (Ici on considère mat comme une image)
La Matrice cle doit être inversible
Il est fortement déconseillé de modifier cette fonction
*/

Matrice Matrice_Decode(Matrice mat, Matrice cle);
/*
Est l'opération inverse de Matrice_Code
Elle permet d'obtenir la matrice envoyée en argument à Matrice_Code avec la matrice renvoyée par celle-ci et la matrice cle
*/

void Matrice_AfficherImage(Matrice mat);
/*
Affiche une matrice sous forme d'image
*/

void Matrice_Afficher(Matrice mat);
/*
Fonction d'affichage utile pour le debug de vos fonctions
*/

int Matrice_MaxTailleCase(Matrice mat);
/*
Fonction utile pour Matrice_Afficher
*/

Matrice Matrice_CreateByArgs(int lignes, int colonnes, ...);
/*
Permet de créer une matrice de taille lignesXcolonnes avec les arguments suivants
*/

#endif
